@@ -55,10 +55,10 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 		_cleanAlgorithm->SetRMSFactorImage(Image());
 	}
 	else {
-		if(!_settings.rmsBackgroundImage.empty())
+		if(!_settings.localRMSImage.empty())
 		{
 			Image rmsImage(_imgWidth, _imgHeight, *_imageAllocator);
-			FitsReader reader(_settings.rmsBackgroundImage);
+			FitsReader reader(_settings.localRMSImage);
 			reader.Read(rmsImage.data());
 			// Normalize the RMS image
 			stddev = rmsImage.Min();
@@ -66,27 +66,33 @@ void Deconvolution::Perform(const class ImagingTable& groupTable, bool& reachedM
 			if(stddev <= 0.0)
 				throw std::runtime_error("RMS image can only contain values > 0, but contains values <= 0.0");
 			for(double& value : rmsImage)
-				value = stddev / value;
+			{
+				if(value != 0.0)
+					value = stddev / value;
+			}
 			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
 		}
-		else if(_settings.rmsBackground)
+		else if(_settings.localRMS)
 		{
 			Image rmsImage;
 			// TODO this should use full beam parameters
-			switch(_settings.rmsBackgroundMethod)
+			switch(_settings.localRMSMethod)
 			{
 				case WSCleanSettings::RMSWindow:
-					RMSImage::Make(rmsImage, integrated, _settings.rmsBackgroundWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
+					RMSImage::Make(rmsImage, integrated, _settings.localRMSWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
 					break;
 				case WSCleanSettings::RMSAndMinimumWindow:
-					RMSImage::MakeWithNegativityLimit(rmsImage, integrated, _settings.rmsBackgroundWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
+					RMSImage::MakeWithNegativityLimit(rmsImage, integrated, _settings.localRMSWindow, _beamSize, _beamSize, 0.0, _pixelScaleX, _pixelScaleY);
 					break;
 			}
 			// Normalize the RMS image relative to the threshold so that Jy remains Jy.
 			stddev = rmsImage.Min();
 			Logger::Info << "Lowest RMS in image: " << FluxDensity::ToNiceString(stddev) << '\n';
 			for(double& value : rmsImage)
-				value = stddev / value;
+			{
+				if(value != 0.0)
+					value = stddev / value;
+			}
 			_cleanAlgorithm->SetRMSFactorImage(std::move(rmsImage));
 		}
 	}
@@ -173,6 +179,12 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 	if(_summedCount == 0)
 		throw std::runtime_error("Nothing to clean");
 	
+	if(!std::isfinite(_beamSize))
+	{
+		Logger::Warn << "No proper beam size available in deconvolution!\n";
+		_beamSize = 0.0;
+	}
+	
 	ImagingTable firstSquaredGroup = groupTable.GetSquaredGroup(0);
 	_squaredCount = firstSquaredGroup.EntryCount();
 	_polarizations.clear();
@@ -228,18 +240,27 @@ void Deconvolution::InitializeDeconvolutionAlgorithm(const ImagingTable& groupTa
 	
 	if(!_settings.fitsDeconvolutionMask.empty())
 	{
-		if(_cleanMask.empty())
+		FitsReader maskReader(_settings.fitsDeconvolutionMask, true, true);
+		if(maskReader.ImageWidth() != _imgWidth || maskReader.ImageHeight() != _imgHeight)
+			throw std::runtime_error("Specified Fits file mask did not have same dimensions as output image!");
+		ao::uvector<float> maskData(_imgWidth*_imgHeight);
+		if(maskReader.NFrequencies() == 1)
 		{
-			Logger::Info << "Reading mask '" << _settings.fitsDeconvolutionMask << "'...\n";
-			FitsReader maskReader(_settings.fitsDeconvolutionMask);
-			if(maskReader.ImageWidth() != _imgWidth || maskReader.ImageHeight() != _imgHeight)
-				throw std::runtime_error("Specified Fits file mask did not have same dimensions as output image!");
-			ao::uvector<float> maskData(_imgWidth*_imgHeight);
+			Logger::Debug << "Reading mask '" << _settings.fitsDeconvolutionMask << "'...\n";
 			maskReader.Read(maskData.data());
-			_cleanMask.assign(_imgWidth*_imgHeight, false);
-			for(size_t i=0; i!=_imgWidth*_imgHeight; ++i)
-				_cleanMask[i] = maskData[i]!=0.0;
 		}
+		else if(maskReader.NFrequencies()==_settings.channelsOut) {
+			Logger::Debug << "Reading mask '" << _settings.fitsDeconvolutionMask << "' (" << (groupTable.Front().outputChannelIndex+1) << ")...\n";
+			maskReader.ReadFrequency(maskData.data(), groupTable.Front().outputChannelIndex);
+		}
+		else {
+			std::stringstream msg;
+			msg << "The number of frequencies in the specified fits mask (" << maskReader.NFrequencies() << ") does not match the number of requested output channels (" << _settings.channelsOut << ")";
+			throw std::runtime_error(msg.str());
+		}
+		_cleanMask.assign(_imgWidth*_imgHeight, false);
+		for(size_t i=0; i!=_imgWidth*_imgHeight; ++i)
+			_cleanMask[i] = (maskData[i]!=0.0);
 		_cleanAlgorithm->SetCleanMask(_cleanMask.data());
 	} else if(!_settings.casaDeconvolutionMask.empty())
 	{
@@ -294,6 +315,49 @@ void Deconvolution::SaveSourceList(const class ImagingTable& table, long double
 	}
 }
 
+void Deconvolution::correctChannelForPB(ComponentList& list, const ImagingTableEntry& entry) const
+{
+	Logger::Debug << "Correcting source list of channel " << entry.outputChannelIndex << " for beam\n";
+	ImageFilename filename(entry.outputChannelIndex, entry.outputIntervalIndex);
+	filename.SetPolarization(entry.polarization);
+	PrimaryBeam pb(_settings);
+	PrimaryBeamImageSet beam(_imgWidth, _imgHeight, *_imageAllocator);
+	pb.Load(beam, filename);
+	list.CorrectForBeam(beam, entry.outputChannelIndex);
+}
+
+void Deconvolution::loadAveragePrimaryBeam(PrimaryBeamImageSet& beamImages, size_t imageIndex, const ImagingTable& table) const
+{
+	Logger::Debug << "Averaging beam for deconvolution channel " << imageIndex << "\n";
+	
+	beamImages.SetToZero();
+	
+	ImageBufferAllocator::Ptr scratch;
+	_imageAllocator->Allocate(_imgWidth*_imgHeight, scratch);
+	size_t deconvolutionChannels = _settings.deconvolutionChannelCount;
+	
+	/// TODO : use real weights of images
+	size_t count = 0;
+	PrimaryBeam pb(_settings);
+	for(size_t sqIndex=0; sqIndex!=table.SquaredGroupCount(); ++sqIndex)
+	{
+		size_t curImageIndex = (sqIndex*deconvolutionChannels)/table.SquaredGroupCount();
+		if(curImageIndex == imageIndex)
+		{
+			const ImagingTableEntry e = table.GetSquaredGroup(sqIndex).Front();
+			Logger::Debug << "Adding beam at " << e.CentralFrequency()*1e-6 << " MHz\n";
+			ImageFilename filename(e.outputChannelIndex, e.outputIntervalIndex);
+			
+			PrimaryBeamImageSet scratch(_settings.trimmedImageWidth, _settings.trimmedImageHeight, *_imageAllocator);
+			pb.Load(scratch, filename);
+			beamImages += scratch;
+			
+			count++;
+		}
+	}
+	beamImages *= (1.0 / double(count));
+}
+
 void Deconvolution::SavePBSourceList(const class ImagingTable& table, long double phaseCentreRA, long double phaseCentreDec) const
 {
 	std::unique_ptr<ComponentList> list;
@@ -309,16 +373,24 @@ void Deconvolution::SavePBSourceList(const class ImagingTable& table, long doubl
 		list.reset(new ComponentList(_imgWidth, _imgHeight, modelSet));
 	}
 	
-	for(size_t i=0; i!=table.SquaredGroupCount(); ++i)
+	if(_settings.deconvolutionChannelCount == 0 ||
+		_settings.deconvolutionChannelCount == table.SquaredGroupCount())
 	{
-		const ImagingTableEntry entry = table.GetSquaredGroup(i).Front();
-		Logger::Debug << "Correcting source list of channel " << entry.outputChannelIndex << " for beam\n";
-		ImageFilename filename(entry.outputChannelIndex, entry.outputIntervalIndex);
-		filename.SetPolarization(entry.polarization);
-		PrimaryBeam pb(_settings);
-		PrimaryBeamImageSet beam(_imgWidth, _imgHeight, *_imageAllocator);
-		pb.Load(beam, filename);
-		list->CorrectForBeam(beam, entry.outputChannelIndex);
+		// No beam averaging is required
+		for(size_t i=0; i!=table.SquaredGroupCount(); ++i)
+		{
+			const ImagingTableEntry entry = table.GetSquaredGroup(i).Front();
+			correctChannelForPB(*list, entry);
+		}
+	}
+	else {
+		for(size_t ch=0; ch!=_settings.deconvolutionChannelCount; ++ch)
+		{
+			PrimaryBeamImageSet beamImages(_imgWidth, _imgHeight, *_imageAllocator);
+			Logger::Debug << "Correcting source list of channel " << ch << " for averaged beam\n";
+			loadAveragePrimaryBeam(beamImages, ch, table);
+			list->CorrectForBeam(beamImages, ch);
+		}
 	}
 	
 	std::string filename = _settings.prefixName + "-sources-pb.txt";
