@@ -225,11 +225,10 @@ void DFTPredictionInput::ConvertApparentToAbsolute(casacore::MeasurementSet& ms)
 
 void DFTPredictionInput::InitializeFromModel(const Model& model, long double phaseCentreRA, long double phaseCentreDec, const BandData& band)
 {
-	for(Model::const_iterator s=model.begin(); s!=model.end(); ++s)
+	for(const ModelSource& s : model)
 	{
-		for(ModelSource::const_iterator c=s->begin(); c!=s->end(); ++c)
+		for(const ModelComponent& comp : s)
 		{
-			const ModelComponent& comp = *c;
 			long double l, m;
 			DFTPredictionComponent& component = AddComponent();
 			ImageCoordinates::RaDecToLM(comp.PosRA(), comp.PosDec(), phaseCentreRA, phaseCentreDec, l, m);
@@ -266,8 +265,9 @@ void DFTPredictionAlgorithm::predict(MC2x2& dest, double u, double v, double w,
 {
 	double l = component.L(), m = component.M(), lmsqrt = component.LMSqrt();
 	double angle = 2.0*M_PI*(u*l + v*m + w*(lmsqrt-1.0));
-	double sinangleOverLMS, cosangleOverLMS;
-	sincos(angle, &sinangleOverLMS, &cosangleOverLMS);
+	double
+		sinangleOverLMS = sin(angle),
+		cosangleOverLMS = cos(angle);
 	MC2x2 appFlux;
 	if(_hasBeam)
 	{
