@@ -33,7 +33,7 @@ class FitsWriter : protected FitsIOChecker
 			_isUV(false),
 			_telescopeName(), _observer(), _objectName(),
 			_origin("AO/WSImager"), _originComment("Imager written by Andre Offringa"),
-			_multiFPtr(0)
+			_multiFPtr(nullptr)
 		{
 		}
 		
@@ -50,14 +50,14 @@ class FitsWriter : protected FitsIOChecker
 			_isUV(false),
 			_telescopeName(), _observer(), _objectName(),
 			_origin("AO/WSImager"), _originComment("Imager written by Andre Offringa"),
-			_multiFPtr(0)
+			_multiFPtr(nullptr)
 		{
 			SetMetadata(reader);
 		}
 		
 		~FitsWriter()
 		{
-			if(_multiFPtr != 0)
+			if(_multiFPtr != nullptr)
 				FinishMulti();
 		}
 		
