@@ -10,9 +10,9 @@ public:
 	: MSRowProvider(msPath, selection, selectedDataDescIds, dataColumnName, requireModel)
 	{ }
 	
-	virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2);
+	virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2, double& time) override;
 	
-	virtual void ReadModel(DataArray& model);
+	virtual void ReadModel(DataArray& model) final override;
 	
 private:
 };
