@@ -6,6 +6,7 @@
 #include "noisemsrowprovider.h"
 
 #include "../progressbar.h"
+#include "../system.h"
 
 #include "../wsclean/logger.h"
 #include "../wsclean/wscleansettings.h"
@@ -16,7 +17,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <string.h>
 
 #include <cstdio>
 #include <fstream>
@@ -34,13 +34,15 @@
 
 PartitionedMS::PartitionedMS(const Handle& handle, size_t partIndex, PolarizationEnum polarization, size_t dataDescId) :
 	_handle(handle),
+	_partIndex(partIndex),
 	_metaFile(getMetaFilename(handle._data->_msPath, handle._data->_temporaryDirectory, dataDescId)),
 	_modelFileMap(0),
 	_currentRow(0),
 	_readPtrIsOk(true),
 	_metaPtrIsOk(true),
 	_weightPtrIsOk(true),
-	_polarization(polarization)
+	_polarization(polarization),
+	_polarizationCountInFile(_polarization==Polarization::Instrumental ? 4 : 1)
 {
 	_metaFile.read(reinterpret_cast<char*>(&_metaHeader), sizeof(MetaHeader));
 	std::vector<char> msPath(_metaHeader.filenameLength+1, char(0));
@@ -61,17 +63,14 @@ PartitionedMS::PartitionedMS(const Handle& handle, size_t partIndex, Polarizatio
 		_fd = open((partPrefix+"-m.tmp").c_str(), O_RDWR);
 		if(_fd == -1)
 			throw std::runtime_error("Error opening temporary model data file");
-		size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount * sizeof(std::complex<float>);
+		size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount * _polarizationCountInFile * sizeof(std::complex<float>);
 		if(length == 0)
 			_modelFileMap = 0;
 		else {
 			_modelFileMap = reinterpret_cast<char*>( mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_NORESERVE, _fd, 0) );
 			if(_modelFileMap == MAP_FAILED)
 			{
-				int errsv = errno;
-				char buffer[1024];
-				const char* msg = strerror_r(errsv, buffer, 1024); 
-				
+				std::string msg = System::StrError(errno); 
 				_modelFileMap = 0;
 				throw std::runtime_error(std::string("Error creating memory map to temporary model file: mmap() returned MAP_FAILED with error message: ") + msg);
 			}
@@ -81,15 +80,15 @@ PartitionedMS::PartitionedMS(const Handle& handle, size_t partIndex, Polarizatio
 	_weightFile.open(partPrefix+"-w.tmp", std::ios::in);
 	if(!_weightFile.good())
 		throw std::runtime_error("Error opening temporary data file");
-	_weightBuffer.resize(_partHeader.channelCount);
-	_modelBuffer.resize(_partHeader.channelCount);
+	_weightBuffer.resize(_partHeader.channelCount * _polarizationCountInFile);
+	_modelBuffer.resize(_partHeader.channelCount * _polarizationCountInFile);
 }
 
 PartitionedMS::~PartitionedMS()
 {
 	if(_modelFileMap!=0)
 	{
-		size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount * sizeof(std::complex<float>);
+		size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount * _polarizationCountInFile * sizeof(std::complex<float>);
 		if(length != 0)
 			munmap(_modelFileMap, length);
 	}
@@ -119,17 +118,17 @@ void PartitionedMS::NextRow()
 	if(_currentRow < _metaHeader.selectedRowCount)
 	{
 		if(_readPtrIsOk)
-			_dataFile.seekg(_partHeader.channelCount * sizeof(std::complex<float>), std::ios::cur);
+			_dataFile.seekg(_partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>), std::ios::cur);
 		else
 			_readPtrIsOk = true;
 		
 		if(_metaPtrIsOk)
-			_metaFile.seekg(sizeof(MetaRecord), std::ios::cur);
+			_metaFile.seekg(MetaRecord::BINARY_SIZE, std::ios::cur);
 		else
 			_metaPtrIsOk = true;
 		
 		if(_weightPtrIsOk && _partHeader.hasWeights)
-			_weightFile.seekg(_partHeader.channelCount * sizeof(float), std::ios::cur);
+			_weightFile.seekg(_partHeader.channelCount * _polarizationCountInFile * sizeof(float), std::ios::cur);
 		_weightPtrIsOk = true;
 	}
 }
@@ -137,38 +136,39 @@ void PartitionedMS::NextRow()
 void PartitionedMS::ReadMeta(double& u, double& v, double& w, size_t& dataDescId)
 {
 	if(!_metaPtrIsOk)
-		_metaFile.seekg(-sizeof(MetaRecord), std::ios::cur);
+		_metaFile.seekg(-MetaRecord::BINARY_SIZE, std::ios::cur);
 	_metaPtrIsOk = false;
 	
 	MetaRecord record;
-	_metaFile.read(reinterpret_cast<char*>(&record), sizeof(MetaRecord));
+	record.read(_metaFile);
 	u = record.u;
 	v = record.v;
 	w = record.w;
 	dataDescId = record.dataDescId;
 }
 
-void PartitionedMS::ReadMeta(double& u, double& v, double& w, size_t& dataDescId, size_t& antenna1, size_t& antenna2)
+void PartitionedMS::ReadMeta(MetaData& metaData)
 {
 	if(!_metaPtrIsOk)
-		_metaFile.seekg(-sizeof(MetaRecord), std::ios::cur);
+		_metaFile.seekg(-MetaRecord::BINARY_SIZE, std::ios::cur);
 	_metaPtrIsOk = false;
 	
 	MetaRecord record;
-	_metaFile.read(reinterpret_cast<char*>(&record), sizeof(MetaRecord));
-	u = record.u;
-	v = record.v;
-	w = record.w;
-	dataDescId = record.dataDescId;
-	antenna1 = record.antenna1;
-	antenna2 = record.antenna2;
+	record.read(_metaFile);
+	metaData.uInM = record.u;
+	metaData.vInM = record.v;
+	metaData.wInM = record.w;
+	metaData.dataDescId = record.dataDescId;
+	metaData.antenna1 = record.antenna1;
+	metaData.antenna2 = record.antenna2;
+	metaData.time = record.time;
 }
 
 void PartitionedMS::ReadData(std::complex<float>* buffer)
 {
 	if(!_readPtrIsOk)
 	{
-		_dataFile.seekg(-_partHeader.channelCount * sizeof(std::complex<float>), std::ios::cur);
+		_dataFile.seekg(-_partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>), std::ios::cur);
 	}
 #ifdef REDUNDANT_VALIDATION
 	size_t pos = size_t(_dataFile.tellg()) - sizeof(PartHeader);
@@ -181,7 +181,7 @@ void PartitionedMS::ReadData(std::complex<float>* buffer)
 		throw std::runtime_error(s.str());
 	}
 #endif
-	_dataFile.read(reinterpret_cast<char*>(buffer), _partHeader.channelCount * sizeof(std::complex<float>));
+	_dataFile.read(reinterpret_cast<char*>(buffer), _partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>));
 	_readPtrIsOk = false;
 }
 
@@ -191,7 +191,7 @@ void PartitionedMS::ReadModel(std::complex<float>* buffer)
 	if(!_partHeader.hasModel)
 		throw std::runtime_error("Partitioned MS initialized without model");
 #endif
-	size_t rowLength = _partHeader.channelCount * sizeof(std::complex<float>);
+	size_t rowLength = _partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>);
 	memcpy(reinterpret_cast<char*>(buffer), _modelFileMap + rowLength*_currentRow, rowLength);
 }
 
@@ -201,38 +201,60 @@ void PartitionedMS::WriteModel(size_t rowId, std::complex<float>* buffer)
 	if(!_partHeader.hasModel)
 		throw std::runtime_error("Partitioned MS initialized without model");
 #endif
-	_weightFile.seekg(_partHeader.channelCount * sizeof(float) * rowId, std::ios::beg);
-	_weightFile.read(reinterpret_cast<char*>(_weightBuffer.data()), _partHeader.channelCount * sizeof(float));
-	for(size_t i=0; i!=_partHeader.channelCount; ++i)
+	_weightFile.seekg(_partHeader.channelCount * _polarizationCountInFile * sizeof(float) * rowId, std::ios::beg);
+	_weightFile.read(reinterpret_cast<char*>(_weightBuffer.data()), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
+	for(size_t i=0; i!=_partHeader.channelCount * _polarizationCountInFile; ++i)
 		buffer[i] *= _weightBuffer[i];
 	
-	size_t rowLength = _partHeader.channelCount * sizeof(std::complex<float>);
+	size_t rowLength = _partHeader.channelCount * _polarizationCountInFile * sizeof(std::complex<float>);
 	std::complex<float>* modelWritePtr = reinterpret_cast<std::complex<float>*>(_modelFileMap + rowLength*rowId);
 	
 	// In case the value was not sampled in this pass, it will be set to infinite and should not overwrite the current
 	// value in the set.
-	for(size_t i=0; i!=_partHeader.channelCount; ++i)
+	for(size_t i=0; i!=_partHeader.channelCount * _polarizationCountInFile; ++i)
 	{
 		if(std::isfinite(buffer[i].real()))
 			modelWritePtr[i] = buffer[i];
 	}
 }
 
+void PartitionedMS::WriteImagingWeights(size_t rowId, const float* buffer)
+{
+	if(_modelDataFile == nullptr)
+	{
+		std::string partPrefix = getPartPrefix(
+			_handle._data->_msPath, _partIndex,
+			_polarization, _partHeader.dataDescId, _handle._data->_temporaryDirectory);
+		_imagingWeightsFile.reset(new std::fstream(partPrefix + "-imgw.tmp",
+		std::ios::in | std::ios::out | std::ios::binary));
+	}
+	const size_t chunkSize = _partHeader.channelCount * _polarizationCountInFile * sizeof(float);
+	_imagingWeightsFile->seekg(chunkSize * rowId, std::ios::beg);
+	_imagingWeightsFile->read(reinterpret_cast<char*>(_imagingWeightBuffer.data()), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
+	for(size_t i=0; i!=_partHeader.channelCount * _polarizationCountInFile; ++i)
+	{
+		if(std::isfinite(buffer[i]))
+			_imagingWeightBuffer[i] = buffer[i];
+	}
+	_imagingWeightsFile->seekp(chunkSize * rowId, std::ios::beg);
+	_imagingWeightsFile->write(reinterpret_cast<const char*>(_imagingWeightBuffer.data()), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
+}
+
 void PartitionedMS::ReadWeights(std::complex<float>* buffer)
 {
 	if(!_weightPtrIsOk)
 		_weightFile.seekg(-_partHeader.channelCount * sizeof(float), std::ios::cur);
-	float* displacedBuffer = reinterpret_cast<float*>(buffer)+_partHeader.channelCount;
-	_weightFile.read(reinterpret_cast<char*>(displacedBuffer), _partHeader.channelCount * sizeof(float));
+	float* displacedBuffer = reinterpret_cast<float*>(buffer)+_partHeader.channelCount * _polarizationCountInFile;
+	_weightFile.read(reinterpret_cast<char*>(displacedBuffer), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
 	_weightPtrIsOk = false;
-	copyRealToComplex(buffer, displacedBuffer, _partHeader.channelCount);
+	copyRealToComplex(buffer, displacedBuffer, _partHeader.channelCount * _polarizationCountInFile);
 }
 
 void PartitionedMS::ReadWeights(float* buffer)
 {
 	if(!_weightPtrIsOk)
-		_weightFile.seekg(-_partHeader.channelCount * sizeof(float), std::ios::cur);
-	_weightFile.read(reinterpret_cast<char*>(buffer), _partHeader.channelCount * sizeof(float));
+		_weightFile.seekg(-_partHeader.channelCount * _polarizationCountInFile * sizeof(float), std::ios::cur);
+	_weightFile.read(reinterpret_cast<char*>(buffer), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
 	_weightPtrIsOk = false;
 }
 
@@ -282,10 +304,10 @@ string PartitionedMS::getMetaFilename(const string& msPathStr, const std::string
 // should be private but is not allowed on older compilers
 struct PartitionFiles
 {
-	std::ofstream
-		*data,
-		*weight,
-		*model;
+	std::unique_ptr<std::ofstream>
+		data,
+		weight,
+		model;
 };
 
 /*
@@ -335,14 +357,14 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	size_t fileIndex = 0;
 	for(size_t part=0; part!=channelParts; ++part)
 	{
-		for(std::set<PolarizationEnum>::const_iterator p=polsOut.begin(); p!=polsOut.end(); ++p)
+		for(PolarizationEnum p : polsOut)
 		{
 			PartitionFiles& f = files[fileIndex];
-			std::string partPrefix = getPartPrefix(msPath, part, *p, channels[part].dataDescId, temporaryDirectory);
-			f.data = new std::ofstream(partPrefix + ".tmp");
-			f.weight = new std::ofstream(partPrefix + "-w.tmp");
+			std::string partPrefix = getPartPrefix(msPath, part, p, channels[part].dataDescId, temporaryDirectory);
+			f.data.reset(new std::ofstream(partPrefix + ".tmp"));
+			f.weight.reset(new std::ofstream(partPrefix + "-w.tmp"));
 			if(initialModelRequired)
-				f.model = new std::ofstream(partPrefix + "-m.tmp");
+				f.model.reset(new std::ofstream(partPrefix + "-m.tmp"));
 			f.data->seekp(sizeof(PartHeader), std::ios::beg);
 			
 			++fileIndex;
@@ -363,7 +385,6 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	std::vector<PolarizationEnum> msPolarizations = GetMSPolarizations(rowProvider->MS());
 	
 	const casacore::IPosition shape(rowProvider->DataShape());
-	const size_t polarizationCount = shape[0];
 	size_t channelCount = shape[1];
 	
 	Logger::Info << "Reordering " << msPath << " into " << channelParts << " x " << polsOut.size() << " parts.\n";
@@ -389,8 +410,8 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	
 	// Write actual data
 	size_t polarizationsPerFile = settings.useIDG ? 4 : 1;
-	std::vector<std::complex<float>> dataBuffer(polarizationCount * channelCount);
-	std::vector<float> weightBuffer(polarizationCount * channelCount);
+	std::vector<std::complex<float>> dataBuffer(polarizationsPerFile * channelCount);
+	std::vector<float> weightBuffer(polarizationsPerFile * channelCount);
 	
 	casacore::Array<std::complex<float>> dataArray(shape), modelArray(shape);
 	casacore::Array<float> weightSpectrumArray(shape);
@@ -406,16 +427,18 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 		MetaRecord meta;
 		memset(&meta, 0, sizeof(MetaRecord));
 
+		double time;
 		uint32_t dataDescId, antenna1, antenna2;
-		rowProvider->ReadData(dataArray, flagArray, weightSpectrumArray, meta.u, meta.v, meta.w, dataDescId, antenna1, antenna2);
+		rowProvider->ReadData(dataArray, flagArray, weightSpectrumArray, meta.u, meta.v, meta.w, dataDescId, antenna1, antenna2, time);
 		meta.dataDescId = dataDescId;
 		meta.antenna1 = antenna1;
 		meta.antenna2 = antenna2;
+		meta.time = time;
 		size_t spwIndex = selectedDataDescIds[meta.dataDescId];
 		++selectedRowCountPerSpwIndex[spwIndex];
 		++selectedRowsTotal;
 		std::ofstream& metaFile = *metaFiles[spwIndex];
-		metaFile.write(reinterpret_cast<char*>(&meta), sizeof(MetaRecord));
+		meta.write(metaFile);
 		if(!metaFile.good())
 			throw std::runtime_error("Error writing to temporary file");
 		
@@ -431,23 +454,23 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 					partStartCh = channels[part].start,
 					partEndCh = channels[part].end;
 				
-				for(std::set<PolarizationEnum>::const_iterator p=polsOut.begin(); p!=polsOut.end(); ++p)
+				for(PolarizationEnum p : polsOut)
 				{
 					PartitionFiles& f = files[fileIndex];
-					copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, *p);
+					copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, p);
 					f.data->write(reinterpret_cast<char*>(dataBuffer.data()), (partEndCh - partStartCh) * sizeof(std::complex<float>) * polarizationsPerFile);
 					if(!f.data->good())
 						throw std::runtime_error("Error writing to temporary data file");
 					
 					if(initialModelRequired)
 					{
-						copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, modelArray, weightSpectrumArray, flagArray, *p);
+						copyWeightedData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations, modelArray, weightSpectrumArray, flagArray, p);
 						f.model->write(reinterpret_cast<char*>(dataBuffer.data()), (partEndCh - partStartCh) * sizeof(std::complex<float>) * polarizationsPerFile);
 						if(!f.model->good())
 							throw std::runtime_error("Error writing to temporary data file");
 					}
 					
-					copyWeights(weightBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, *p);
+					copyWeights(weightBuffer.data(), partStartCh, partEndCh, msPolarizations, dataArray, weightSpectrumArray, flagArray, p);
 					f.weight->write(reinterpret_cast<char*>(weightBuffer.data()), (partEndCh - partStartCh) * sizeof(float) * polarizationsPerFile);
 					if(!f.weight->good())
 						throw std::runtime_error("Error writing to temporary weights file");
@@ -485,7 +508,7 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	header.hasModel = includeModel;
 	header.hasWeights = true;
 	fileIndex = 0;
-	dataBuffer.assign(channelCount, 0.0);
+	dataBuffer.assign(channelCount * polarizationsPerFile, 0.0);
 	std::unique_ptr<ProgressBar> progress2;
 	if(includeModel && !initialModelRequired)
 		progress2.reset(new ProgressBar("Initializing model visibilities"));
@@ -502,10 +525,9 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 			if(!f.data->good())
 				throw std::runtime_error("Error writing to temporary data file");
 			
-			delete f.data;
-			delete f.weight;
-			if(initialModelRequired)
-				delete f.model;
+			f.data.reset();
+			f.weight.reset();
+			f.model.reset();
 			++fileIndex;
 			
 			// If model is requested, fill model file with zeros
@@ -527,25 +549,25 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	return Handle(msPath, dataColumnName, temporaryDirectory, channels, initialModelRequired, modelUpdateRequired, polsOut, selection);
 }
 
-void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
+void PartitionedMS::unpartition(const PartitionedMS::Handle::HandleData& handle)
 {
-	const std::set<PolarizationEnum> pols = handle._data->_polarizations;
+	const std::set<PolarizationEnum> pols = handle._polarizations;
 	
 	std::map<size_t,size_t> dataDescIds;
-	getDataDescIdMap(dataDescIds, handle._data->_channels);
+	getDataDescIdMap(dataDescIds, handle._channels);
 	
 	std::vector<MetaHeader> metaHeaders(dataDescIds.size());
 	for(const std::pair<size_t,size_t>& dataDescId : dataDescIds)
 	{
-		std::ifstream metaFile(getMetaFilename(handle._data->_msPath, handle._data->_temporaryDirectory, dataDescId.first));
+		std::ifstream metaFile(getMetaFilename(handle._msPath, handle._temporaryDirectory, dataDescId.first));
 		MetaHeader& metaHeader = metaHeaders[dataDescId.second];
 		metaFile.read(reinterpret_cast<char*>(&metaHeader), sizeof(MetaHeader));
 		std::vector<char> msPath(metaHeader.filenameLength+1, char(0));
 		metaFile.read(msPath.data(), metaHeader.filenameLength);
 	}
 	
-	ChannelRange firstRange = handle._data->_channels[0];
-	std::ifstream firstDataFile(getPartPrefix(handle._data->_msPath, 0, *pols.begin(), firstRange.dataDescId, handle._data->_temporaryDirectory)+".tmp", std::ios::in);
+	ChannelRange firstRange = handle._channels[0];
+	std::ifstream firstDataFile(getPartPrefix(handle._msPath, 0, *pols.begin(), firstRange.dataDescId, handle._temporaryDirectory)+".tmp", std::ios::in);
 	if(!firstDataFile.good())
 		throw std::runtime_error("Error opening temporary data file");
 	PartHeader firstPartHeader;
@@ -555,17 +577,17 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 	
 	if(firstPartHeader.hasModel)
 	{
-		const size_t channelParts = handle._data->_channels.size();
+		const size_t channelParts = handle._channels.size();
 		
 		// Open the temporary files
 		std::vector<std::ifstream*> modelFiles(channelParts*pols.size()), weightFiles(channelParts*pols.size());
 		size_t fileIndex = 0;
 		for(size_t part=0; part!=channelParts; ++part)
 		{
-			size_t dataDescId = handle._data->_channels[part].dataDescId;
+			size_t dataDescId = handle._channels[part].dataDescId;
 			for(std::set<PolarizationEnum>::const_iterator p=pols.begin(); p!=pols.end(); ++p)
 			{
-				std::string partPrefix = getPartPrefix(handle._data->_msPath, part, *p, dataDescId, handle._data->_temporaryDirectory);
+				std::string partPrefix = getPartPrefix(handle._msPath, part, *p, dataDescId, handle._temporaryDirectory);
 				modelFiles[fileIndex] = new std::ifstream(partPrefix + "-m.tmp");
 				if(firstPartHeader.hasWeights)
 					weightFiles[fileIndex] = new std::ifstream(partPrefix + "-w.tmp");
@@ -573,7 +595,7 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 			}
 		}
 		
-		casacore::MeasurementSet ms(handle._data->_msPath, casacore::Table::Update);
+		casacore::MeasurementSet ms(handle._msPath, casacore::Table::Update);
 		const std::vector<PolarizationEnum> msPolarizations = GetMSPolarizations(ms);
 		initializeModelColumn(ms);
 		casacore::ROScalarColumn<int> antenna1Column(ms, ms.columnName(casacore::MSMainEnums::ANTENNA1));
@@ -581,21 +603,22 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 		casacore::ROScalarColumn<int> fieldIdColumn(ms, ms.columnName(casacore::MSMainEnums::FIELD_ID));
 		casacore::ROScalarColumn<double> timeColumn(ms, ms.columnName(casacore::MSMainEnums::TIME));
 		casacore::ROScalarColumn<int> dataDescIdColumn(ms, ms.columnName(casacore::MSMainEnums::DATA_DESC_ID));
-		casacore::ROArrayColumn<casacore::Complex> dataColumn(ms, handle._data->_dataColumnName);
+		casacore::ROArrayColumn<casacore::Complex> dataColumn(ms, handle._dataColumnName);
 		casacore::ArrayColumn<casacore::Complex> modelColumn(ms, ms.columnName(casacore::MSMainEnums::MODEL_DATA));
 		casacore::ROArrayColumn<double> uvwColumn(ms, ms.columnName(casacore::MSMainEnums::UVW));
 		
 		const casacore::IPosition shape(dataColumn.shape(0));
 		size_t channelCount = shape[1];
 		
-		std::vector<std::complex<float>> modelDataBuffer(channelCount);
-		std::vector<float> weightBuffer(channelCount);
+		size_t polarizationsPerFile = (*pols.begin())==Polarization::Instrumental ? 4 : 1;
+		std::vector<std::complex<float>> modelDataBuffer(channelCount * polarizationsPerFile);
+		std::vector<float> weightBuffer(channelCount * polarizationsPerFile);
 		casacore::Array<std::complex<float>> modelDataArray(shape);
 	
-		ProgressBar progress(std::string("Writing changed model back to ") + handle._data->_msPath);
+		ProgressBar progress(std::string("Writing changed model back to ") + handle._msPath);
 		size_t startRow, endRow;
-		getRowRange(ms, handle._data->_selection, startRow, endRow);
-		size_t timestep = handle._data->_selection.HasInterval() ? handle._data->_selection.IntervalStart() : 0;
+		getRowRange(ms, handle._selection, startRow, endRow);
+		size_t timestep = handle._selection.HasInterval() ? handle._selection.IntervalStart() : 0;
 		double time = timeColumn(startRow);
 		size_t selectedRowCountForDebug = 0;
 		for(size_t row=startRow; row!=endRow; ++row)
@@ -611,7 +634,7 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 				++timestep;
 				time = timeColumn(row);
 			}
-			if(handle._data->_selection.IsSelected(fieldId, timestep, a1, a2, uvw))
+			if(handle._selection.IsSelected(fieldId, timestep, a1, a2, uvw))
 			{
 				std::map<size_t,size_t>::const_iterator dataDescIdIter = dataDescIds.find(dataDescId);
 				if(dataDescIdIter != dataDescIds.end())
@@ -621,22 +644,22 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 					for(size_t part=0; part!=channelParts; ++part)
 					{
 						size_t
-							dataDescId = handle._data->_channels[part].dataDescId,
-							partStartCh = handle._data->_channels[part].start,
-							partEndCh = handle._data->_channels[part].end;
+							dataDescId = handle._channels[part].dataDescId,
+							partStartCh = handle._channels[part].start,
+							partEndCh = handle._channels[part].end;
 						if(dataDescId == dataDescIdIter->second)
 						{
 							for(std::set<PolarizationEnum>::const_iterator p=pols.begin(); p!=pols.end(); ++p)
 							{
-								modelFiles[fileIndex]->read(reinterpret_cast<char*>(modelDataBuffer.data()), (partEndCh - partStartCh) * sizeof(std::complex<float>));
+								modelFiles[fileIndex]->read(reinterpret_cast<char*>(modelDataBuffer.data()), (partEndCh - partStartCh) * polarizationsPerFile * sizeof(std::complex<float>));
 								if(!modelFiles[fileIndex]->good())
 									throw std::runtime_error("Error reading from temporary model data file");
 								if(firstPartHeader.hasWeights)
 								{
-									weightFiles[fileIndex]->read(reinterpret_cast<char*>(weightBuffer.data()), (partEndCh - partStartCh) * sizeof(float));
+									weightFiles[fileIndex]->read(reinterpret_cast<char*>(weightBuffer.data()), (partEndCh - partStartCh) * polarizationsPerFile * sizeof(float));
 									if(!weightFiles[fileIndex]->good())
 										throw std::runtime_error("Error reading from temporary weight data file");
-									for(size_t i=0; i!=partEndCh - partStartCh; ++i)
+									for(size_t i=0; i!=(partEndCh - partStartCh) * polarizationsPerFile; ++i)
 										modelDataBuffer[i] /= weightBuffer[i];
 								}
 								reverseCopyData(modelDataArray, partStartCh, partEndCh, msPolarizations, modelDataBuffer.data(), *p);
@@ -671,35 +694,30 @@ void PartitionedMS::unpartition(const PartitionedMS::Handle& handle)
 	}
 }
 
-void PartitionedMS::Handle::decrease()
+PartitionedMS::Handle::HandleData::~HandleData()
 {
-	--(_data->_referenceCount);
-	if(_data->_referenceCount == 0)
+	if(_modelUpdateRequired && !_initialModelRequired)
+		PartitionedMS::unpartition(*this);
+	
+	Logger::Info << "Cleaning up temporary files...\n";
+	
+	std::set<size_t> removedMetaFiles;
+	for(size_t part=0; part!=_channels.size(); ++part)
 	{
-		if(_data->_modelUpdateRequired && !_data->_initialModelRequired)
-			PartitionedMS::unpartition(*this);
-		
-		Logger::Info << "Cleaning up temporary files...\n";
-		
-		std::set<size_t> removedMetaFiles;
-		for(size_t part=0; part!=_data->_channels.size(); ++part)
+		for(PolarizationEnum p : _polarizations)
 		{
-			for(std::set<PolarizationEnum>::const_iterator p=_data->_polarizations.begin(); p!=_data->_polarizations.end(); ++p)
-			{
-				std::string prefix = getPartPrefix(_data->_msPath, part, *p, _data->_channels[part].dataDescId, _data->_temporaryDirectory);
-				std::remove((prefix + ".tmp").c_str());
-				std::remove((prefix + "-w.tmp").c_str());
-				std::remove((prefix + "-m.tmp").c_str());
-			}
-			size_t dataDescId = _data->_channels[part].dataDescId;
-			if(removedMetaFiles.count(dataDescId) == 0)
-			{
-				removedMetaFiles.insert(dataDescId);
-				std::string metaFile = getMetaFilename(_data->_msPath, _data->_temporaryDirectory, dataDescId);
-				std::remove(metaFile.c_str());
-			}
+			std::string prefix = getPartPrefix(_msPath, part, p, _channels[part].dataDescId, _temporaryDirectory);
+			std::remove((prefix + ".tmp").c_str());
+			std::remove((prefix + "-w.tmp").c_str());
+			std::remove((prefix + "-m.tmp").c_str());
+		}
+		size_t dataDescId = _channels[part].dataDescId;
+		if(removedMetaFiles.count(dataDescId) == 0)
+		{
+			removedMetaFiles.insert(dataDescId);
+			std::string metaFile = getMetaFilename(_msPath, _temporaryDirectory, dataDescId);
+			std::remove(metaFile.c_str());
 		}
-		delete _data;
 	}
 }
 
