@@ -28,6 +28,8 @@ MultiScaleAlgorithm::MultiScaleAlgorithm(ImageBufferAllocator& allocator, double
 	_trackPerScaleMasks(false), _usePerScaleMasks(false),
 	_fastSubMinorLoop(true), _trackComponents(false)
 {
+	if(_beamSizeInPixels<=0.0)
+		_beamSizeInPixels = 1;
 }
 
 MultiScaleAlgorithm::~MultiScaleAlgorithm()
@@ -115,7 +117,12 @@ void MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& mo
 	
 	size_t scaleWithPeak;
 	findActiveScaleConvolvedMaxima(dirtySet, integratedScratch.data(), scratch.data(), true);
-	sortScalesOnMaxima(scaleWithPeak);
+	if(!selectMaximumScale(scaleWithPeak))
+	{
+		Logger::Warn << "No peak found during multi-scale cleaning! Aborting deconvolution.\n";
+		reachedMajorThreshold = false;
+		return;
+	}
 	
 	double mGainThreshold = std::fabs(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor) * (1.0 - _mGain);
 	const double firstThreshold = std::max(_threshold, mGainThreshold);
@@ -272,7 +279,7 @@ void MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& mo
 		activateScales(scaleWithPeak);
 		
 		findActiveScaleConvolvedMaxima(dirtySet, integratedScratch.data(), scratch.data(), false);
-		sortScalesOnMaxima(scaleWithPeak);
+		selectMaximumScale(scaleWithPeak);
 		
 		Logger::Info << "Iteration " << _iterationNumber << ", scale " << round(_scaleInfos[scaleWithPeak].scale) << " px : " << FluxDensity::ToNiceString(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue*_scaleInfos[scaleWithPeak].biasFactor) << " at " << _scaleInfos[scaleWithPeak].maxImageValueX << ',' << _scaleInfos[scaleWithPeak].maxImageValueY << '\n';
 	}
@@ -414,8 +421,8 @@ void MultiScaleAlgorithm::findActiveScaleConvolvedMaxima(const ImageSet& imageSe
 	for(size_t i=0; i!=results.size(); ++i)
 	{
 		ScaleInfo& scaleEntry = _scaleInfos[transformIndices[i]];
-		scaleEntry.maxNormalizedImageValue = results[i].normalizedValue;
-		scaleEntry.maxUnnormalizedImageValue = results[i].unnormalizedValue;
+		scaleEntry.maxNormalizedImageValue = get_optional_value_or(results[i].normalizedValue, 0.0);
+		scaleEntry.maxUnnormalizedImageValue = get_optional_value_or(results[i].unnormalizedValue, 0.0);
 		scaleEntry.maxImageValueX = results[i].x;
 		scaleEntry.maxImageValueY = results[i].y;
 		if(reportRMS)
@@ -440,7 +447,7 @@ void MultiScaleAlgorithm::findActiveScaleConvolvedMaxima(const ImageSet& imageSe
 	}
 }
 
-void MultiScaleAlgorithm::sortScalesOnMaxima(size_t& scaleWithPeak)
+bool MultiScaleAlgorithm::selectMaximumScale(size_t& scaleWithPeak)
 {
 	// Find max component
 	std::map<double,size_t> peakToScaleMap;
@@ -449,25 +456,19 @@ void MultiScaleAlgorithm::sortScalesOnMaxima(size_t& scaleWithPeak)
 		if(_scaleInfos[i].isActive)
 		{
 			double maxVal = std::fabs(_scaleInfos[i].maxUnnormalizedImageValue * _scaleInfos[i].biasFactor);
-			if(std::isfinite(maxVal))
-				peakToScaleMap.insert(std::make_pair(maxVal, i));
+			peakToScaleMap.insert(std::make_pair(maxVal, i));
 		}
 	}
 	if(peakToScaleMap.empty())
 	{
-		Logger::Warn << "No scale found with a peak!\n";
 		scaleWithPeak = size_t(-1);
+		return false;
 	}
 	else {
 		std::map<double,size_t>::const_reverse_iterator mapIter = peakToScaleMap.rbegin();
 		scaleWithPeak = mapIter->second;
+		return true;
 	}
-	//++mapIter;
-	//size_t runnerUp;
-	//if(mapIter != peakToScaleMap.rend())
-	//	runnerUp = mapIter->second;
-	//else
-	//	runnerUp = scaleWithPeak;
 }
 
 void MultiScaleAlgorithm::activateScales(size_t scaleWithLastPeak)
@@ -540,7 +541,7 @@ void MultiScaleAlgorithm::findPeakDirect(const double* image, double* scratch, s
 		actualImage = scratch;
 	}
 	
-	double maxValue;
+	boost::optional<double> maxValue;
 	if(_usePerScaleMasks)
 		maxValue = SimpleClean::FindPeakWithMask(actualImage, _width, _height, scaleInfo.maxImageValueX, scaleInfo.maxImageValueY, _allowNegativeComponents, 0, _height, _scaleMasks[scaleIndex].data(), horBorderSize, vertBorderSize);
 	else if(_cleanMask == 0)
@@ -548,9 +549,9 @@ void MultiScaleAlgorithm::findPeakDirect(const double* image, double* scratch, s
 	else
 		maxValue = SimpleClean::FindPeakWithMask(actualImage, _width, _height, scaleInfo.maxImageValueX, scaleInfo.maxImageValueY, _allowNegativeComponents, 0, _height, _cleanMask, horBorderSize, vertBorderSize);
 	
-	scaleInfo.maxUnnormalizedImageValue = maxValue;
+	scaleInfo.maxUnnormalizedImageValue = get_optional_value_or(maxValue, 0.0);
 	if(_rmsFactorImage.empty())
-		scaleInfo.maxNormalizedImageValue = maxValue;
+		scaleInfo.maxNormalizedImageValue = get_optional_value_or(maxValue, 0.0);
 	else
-		scaleInfo.maxNormalizedImageValue = maxValue / _rmsFactorImage[scaleInfo.maxImageValueX + scaleInfo.maxImageValueY * _width];
+		scaleInfo.maxNormalizedImageValue = get_optional_value_or(maxValue, 0.0) / _rmsFactorImage[scaleInfo.maxImageValueX + scaleInfo.maxImageValueY * _width];
 }
