@@ -123,50 +123,59 @@ public:
 	static void SaveSkyModel(const std::string& destination, const Model& model, bool convertClustersToPatches)
 	{
 		std::ofstream file(destination);
-		file << "# (Name, Patch, Type, Ra, Dec, I, Q, U, V, ReferenceFrequency='150.e6', SpectralIndex) = format\n\n";
+		file.precision(15);
+		file << "Format = Name, Patch, Type, Ra, Dec, I, Q, U, V, SpectralIndex, LogarithmicSI, ReferenceFrequency='150.e6', MajorAxis, MinorAxis, Orientation\n\n";
 		std::string patchName = ",";
-		for(Model::const_iterator s=model.begin(); s!=model.end(); ++s)
+		for(const ModelSource& source : model)
 		{
 			// Define a patch for this source
 			// A patch is created by not giving a source name
 			if(convertClustersToPatches)
 			{
-				if(patchName != s->ClusterName())
+				if(patchName != source.ClusterName())
 				{
-					patchName = s->ClusterName();
-					file << ", " << s->ClusterName() << ", POINT, , , , , , , ,\n";
+					patchName = source.ClusterName();
+					file << ", " << source.ClusterName() << ", POINT, , , , , , , , , , , ,\n";
 				}
 			}
 			else {
-				file << ", " << s->Name() << ", POINT, , , , , , , ,\n";
-				patchName = s->Name();
+				file << ", " << source.Name() << ", POINT, , , , , , , , , , , ,\n";
+				patchName = source.Name();
 			}
 					
-			for(size_t ci=0; ci!=s->ComponentCount(); ++ci)
+			for(size_t ci=0; ci!=source.ComponentCount(); ++ci)
 			{
-				const ModelComponent& c = s->Component(ci);
-				file << s->Name() << '_' << ci << ", " << patchName << ", POINT, "
-					<< RaDecCoord::RAToString(c.PosRA(), ':') << ", "
+				const ModelComponent& c = source.Component(ci);
+				file << source.Name();
+				if(source.ComponentCount()>1)
+					file << '_' << ci;
+				file << ", " << patchName << ", ";
+				if(c.Type() == ModelComponent::GaussianSource)
+					file << "GAUSSIAN, ";
+				else
+					file << "POINT, ";
+				file << RaDecCoord::RAToString(c.PosRA(), ':') << ", "
 					<< RaDecCoord::DecToString(c.PosDec(), '.') << ", ";
+				double refFreq;
 				if(c.HasMeasuredSED())
 				{
 					const MeasuredSED& sed = c.MSED();
 					if(sed.MeasurementCount() != 1)
-						throw std::runtime_error("Can only predict single-measurement sky models");
+						throw std::runtime_error("Can only save single-measurement sky models in BBS sky models");
+					refFreq = sed.ReferenceFrequencyHz();
 					double
-						refFreq = sed.ReferenceFrequencyHz(),
 						i = sed.FluxAtFrequency(refFreq, Polarization::StokesI),
 						q = sed.FluxAtFrequency(refFreq, Polarization::StokesQ),
 						u = sed.FluxAtFrequency(refFreq, Polarization::StokesU),
 						v = sed.FluxAtFrequency(refFreq, Polarization::StokesV);
-					file << i << ", " << q << ", " << u << ", " << v << ", " << refFreq << ", []\n";
+					file << i << ", " << q << ", " << u << ", " << v << ", [], false, ";
 				}
 				else {
 					const PowerLawSED& sed = static_cast<const PowerLawSED&>(c.SED());
-					double refFreq, flux[4];
+					double flux[4];
 					std::vector<double> siterms;
 					sed.GetData(refFreq, flux, siterms);
-					file << flux[0] << ", " << flux[1] << ", " << flux[2] << ", " << flux[3] << ", " << refFreq << ", [" ;
+					file << flux[0] << ", " << flux[1] << ", " << flux[2] << ", " << flux[3] << ", [" ;
 					if(siterms.empty())
 						file << "0";
 					else {
@@ -174,8 +183,21 @@ public:
 						for(size_t i=1; i!=siterms.size(); ++i)
 							file << ", " << siterms[i];
 					}
-					file << "]\n";
+					file << "], ";
+					if(sed.IsLogarithmic())
+						file << "true, ";
+					else
+						file << "false, ";
 				}
+				file << refFreq << ", ";
+				if(c.Type() == ModelComponent::GaussianSource)
+				{
+					file
+						<< c.MajorAxis()*(180*3600.0/M_PI) << ", "
+						<< c.MinorAxis()*(180*3600.0/M_PI) << ", " 
+						<< c.PositionAngle()*(180.0/M_PI) << '\n';
+				}
+				else file << ", , ,\n";
 			}
 		}
 	}
