@@ -1,13 +1,16 @@
 #ifndef MS_GRIDDER_BASE_H
 #define MS_GRIDDER_BASE_H
 
-#include "inversionalgorithm.h"
+#include "measurementsetgridder.h"
+
 #include "../multibanddata.h"
+#include "../uvector.h"
 
 class MSGridderBase : public MeasurementSetGridder
 {
 public:
 	MSGridderBase();
+	~MSGridderBase();
 	
 	virtual double StartTime() const final override { return _startTime; }
 	virtual double PhaseCentreRA() const final override { return _phaseCentreRA; }
@@ -15,9 +18,9 @@ public:
 	virtual double PhaseCentreDL() const final override { return _phaseCentreDL; }
 	virtual double PhaseCentreDM() const final override { return _phaseCentreDM; }
 	virtual bool HasDenormalPhaseCentre() const final override { return _denormalPhaseCentre; }
-	virtual double ImageWeight() const final override { return _totalWeight*0.5; }
+	virtual double ImageWeight() const final override { return _totalWeight*2.0; }
 	virtual double NormalizationFactor() const final override {
-		return NormalizeForWeighting() ? _totalWeight*0.5 : 1.0;
+		return NormalizeForWeighting() ? _totalWeight*2.0 : 1.0;
 	}
 	virtual double BeamSize() const final override { return _theoreticalBeamSize; }
 	
@@ -50,6 +53,17 @@ public:
 	const std::string& Observer() const { return _observer; }
 	
 	const std::string& FieldName() const { return _fieldName; }
+	
+	struct MetaDataCache
+	{
+		struct Entry {
+			double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM;
+		};
+		std::vector<Entry> msDataVector;
+	};
+	
+	void SetMetaDataCache(MetaDataCache* cache) { _metaDataCache = cache; }
+	
 protected:
 	struct MSData
 	{
@@ -61,7 +75,7 @@ protected:
 			MultiBandData bandData;
 			size_t startChannel, endChannel;
 			size_t matchingRows, totalRowsProcessed;
-			double minW, maxW, maxBaselineUVW;
+			double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM;
 			size_t rowStart, rowEnd;
 		
 			MultiBandData SelectedBand() const { return MultiBandData(bandData, startChannel, endChannel); }
@@ -70,11 +84,11 @@ protected:
 			
 			void operator=(const MSData &source);
 	};
-		
+
 	struct InversionRow
 	{
 		double uvw[3];
-		size_t dataDescId;
+		size_t dataDescId, rowId;
 		std::complex<float>* data;
 	};
 		
@@ -105,7 +119,7 @@ protected:
 	template<size_t NPolInMSProvider>
 	void calculateWLimits(MSGridderBase::MSData& msData);
 	
-	void initializeMeasurementSet(MSGridderBase::MSData& msData);
+	void initializeMeasurementSet(MSGridderBase::MSData& msData, MetaDataCache::Entry& cacheEntry, bool isCacheInitialized);
 	
 	void calculateOverallMetaData(const MSData* msDataVector);
 	
@@ -127,7 +141,7 @@ protected:
 	
 	double totalWeight() const { return _totalWeight; }
 	
-	void initializeMSDataVector(std::vector<MSData>& msDataVector, size_t nPolInMSProvider);
+	void initializeMSDataVector(std::vector<MSData>& msDataVector);
 	
 private:
 	template<size_t PolarizationCount>
@@ -143,6 +157,7 @@ private:
 	double _freqHigh, _freqLow;
 	double _bandStart, _bandEnd;
 	double _startTime;
+	struct MetaDataCache* _metaDataCache;
 	
 	double _phaseCentreRA, _phaseCentreDec, _phaseCentreDL, _phaseCentreDM;
 	bool _denormalPhaseCentre;
@@ -152,6 +167,8 @@ private:
 	double _totalWeight;
 	double _maxGriddedWeight;
 	double _visibilityWeightSum;
+	
+	ao::uvector<float> _scratchWeights;
 };
 
 #endif
