@@ -1,8 +1,7 @@
 #include "wsclean.h"
 
-#include "binneduvoutput.h"
 #include "imageweightcache.h"
-#include "inversionalgorithm.h"
+#include "measurementsetgridder.h"
 #include "logger.h"
 #include "wscfitswriter.h"
 #include "wsmsgridder.h"
@@ -71,6 +70,8 @@ void WSClean::imagePSF(ImagingTableEntry& entry)
 	_gridder->SetDoImagePSF(true);
 	_gridder->SetDoSubtractModel(false);
 	_gridder->SetVerbose(_isFirstInversion);
+	_gridder->SetMetaDataCache(&_msGridderMetaCache[entry.index]);
+	_gridder->SetStoreImagingWeights(_settings.writeImagingWeightSpectrumColumn);
 	_gridder->Invert();
 	
 	size_t centralIndex = _settings.trimmedImageWidth/2 + (_settings.trimmedImageHeight/2) * _settings.trimmedImageWidth;
@@ -133,42 +134,49 @@ void WSClean::imageGridding()
 	Logger::Info << "DONE\n";
 }
 
-void WSClean::imageMainFirst(PolarizationEnum polarization, size_t joinedChannelIndex)
+void WSClean::imageMainFirst(const ImagingTableEntry& entry)
 {
+	size_t joinedChannelIndex = entry.outputChannelIndex;
+	
 	Logger::Info.Flush();
 	Logger::Info << " == Constructing image ==\n";
 	_inversionWatch.Start();
 	_gridder->SetDoImagePSF(false);
 	_gridder->SetDoSubtractModel(_settings.subtractModel || _settings.continuedRun);
 	_gridder->SetVerbose(_isFirstInversion);
+	_gridder->SetMetaDataCache(&_msGridderMetaCache[entry.index]);
+	_gridder->SetStoreImagingWeights(_settings.writeImagingWeightSpectrumColumn);
 	_gridder->Invert();
 	_inversionWatch.Pause();
 	_gridder->SetVerbose(false);
 	
 	multiplyImage(_infoPerChannel[joinedChannelIndex].psfNormalizationFactor, _gridder->ImageRealResult());
-	storeAndCombineXYandYX(_residualImages, polarization, joinedChannelIndex, false, _gridder->ImageRealResult());
-	if(Polarization::IsComplex(polarization))
+	storeAndCombineXYandYX(_residualImages, entry.polarization, joinedChannelIndex, false, _gridder->ImageRealResult());
+	if(Polarization::IsComplex(entry.polarization))
 	{
 		multiplyImage(_infoPerChannel[joinedChannelIndex].psfNormalizationFactor, _gridder->ImageImaginaryResult());
-		storeAndCombineXYandYX(_residualImages, polarization, joinedChannelIndex, true, _gridder->ImageImaginaryResult());
+		storeAndCombineXYandYX(_residualImages, entry.polarization, joinedChannelIndex, true, _gridder->ImageImaginaryResult());
 	}
 }
 
-void WSClean::imageMainNonFirst(PolarizationEnum polarization, size_t joinedChannelIndex)
+void WSClean::imageMainNonFirst(const ImagingTableEntry& entry)
 {
+	size_t joinedChannelIndex = entry.outputChannelIndex;
+	
 	Logger::Info.Flush();
 	Logger::Info << " == Constructing image ==\n";
 	_inversionWatch.Start();
 	_gridder->SetDoSubtractModel(true);
+	_gridder->SetMetaDataCache(&_msGridderMetaCache[entry.index]);
 	_gridder->Invert();
 	_inversionWatch.Pause();
 	
 	multiplyImage(_infoPerChannel[joinedChannelIndex].psfNormalizationFactor, _gridder->ImageRealResult());
-	storeAndCombineXYandYX(_residualImages, polarization, joinedChannelIndex, false, _gridder->ImageRealResult());
-	if(Polarization::IsComplex(polarization))
+	storeAndCombineXYandYX(_residualImages, entry.polarization, joinedChannelIndex, false, _gridder->ImageRealResult());
+	if(Polarization::IsComplex(entry.polarization))
 	{
 		multiplyImage(_infoPerChannel[joinedChannelIndex].psfNormalizationFactor, _gridder->ImageImaginaryResult());
-		storeAndCombineXYandYX(_residualImages, polarization, joinedChannelIndex, true, _gridder->ImageImaginaryResult());
+		storeAndCombineXYandYX(_residualImages, entry.polarization, joinedChannelIndex, true, _gridder->ImageImaginaryResult());
 	}
 }
 
@@ -198,7 +206,7 @@ void WSClean::storeAndCombineXYandYX(CachedImageSet& dest, PolarizationEnum pola
 	}
 }
 
-void WSClean::predict(PolarizationEnum polarization, size_t joinedChannelIndex)
+void WSClean::predict(const ImagingTableEntry& entry)
 {
 	Logger::Info.Flush();
 	Logger::Info << " == Converting model image to visibilities ==\n";
@@ -207,26 +215,27 @@ void WSClean::predict(PolarizationEnum polarization, size_t joinedChannelIndex)
 		*modelImageReal = _imageAllocator.Allocate(size),
 		*modelImageImaginary = 0;
 		
-	if(polarization == Polarization::YX)
+	if(entry.polarization == Polarization::YX)
 	{
-		_modelImages.Load(modelImageReal, Polarization::XY, joinedChannelIndex, false);
+		_modelImages.Load(modelImageReal, Polarization::XY, entry.outputChannelIndex, false);
 		modelImageImaginary = _imageAllocator.Allocate(size);
-		_modelImages.Load(modelImageImaginary, Polarization::XY, joinedChannelIndex, true);
+		_modelImages.Load(modelImageImaginary, Polarization::XY, entry.outputChannelIndex, true);
 		for(size_t i=0; i!=size; ++i)
 			modelImageImaginary[i] = -modelImageImaginary[i];
 	}
 	else {
-		_modelImages.Load(modelImageReal, polarization, joinedChannelIndex, false);
-		if(Polarization::IsComplex(polarization))
+		_modelImages.Load(modelImageReal, entry.polarization, entry.outputChannelIndex, false);
+		if(Polarization::IsComplex(entry.polarization))
 		{
 			modelImageImaginary = _imageAllocator.Allocate(size);
-			_modelImages.Load(modelImageImaginary, polarization, joinedChannelIndex, true);
+			_modelImages.Load(modelImageImaginary, entry.polarization, entry.outputChannelIndex, true);
 		}
 	}
 	
 	_predictingWatch.Start();
 	_gridder->SetAddToModel(false);
-	if(Polarization::IsComplex(polarization))
+	_gridder->SetMetaDataCache(&_msGridderMetaCache[entry.index]);
+	if(Polarization::IsComplex(entry.polarization))
 		_gridder->Predict(modelImageReal, modelImageImaginary);
 	else
 		_gridder->Predict(modelImageReal);
@@ -337,7 +346,7 @@ void WSClean::initializeMFSImageWeights()
 			for(size_t d=0; d!=_msBands[i].DataDescCount(); ++d)
 			{
 				PolarizationEnum pol = _settings.useIDG ? Polarization::Instrumental : *_settings.polarizations.begin();
-				ContiguousMS msProvider(_settings.filenames[i], _settings.dataColumnName, _globalSelection, pol, d, _settings.deconvolutionMGain != 1.0);
+				ContiguousMS msProvider(_settings.filenames[i], _settings.dataColumnName, _globalSelection, pol, d);
 				_imageWeightCache->Weights().Grid(msProvider,  _globalSelection);
 				Logger::Info << '.';
 				Logger::Info.Flush();
@@ -353,8 +362,8 @@ void WSClean::initializeMFSImageWeights()
 void WSClean::prepareInversionAlgorithm(PolarizationEnum polarization)
 {
 	_gridder->SetGridMode(_settings.gridMode);
-	_gridder->SetImageWidth(_settings.untrimmedImageWidth);
-	_gridder->SetImageHeight(_settings.untrimmedImageHeight);
+	_gridder->SetImageWidth(_settings.paddedImageWidth);
+	_gridder->SetImageHeight(_settings.paddedImageHeight);
 	_gridder->SetTrimSize(_settings.trimmedImageWidth, _settings.trimmedImageHeight);
 	_gridder->SetNWSize(_settings.widthForNWCalculation, _settings.heightForNWCalculation);
 	_gridder->SetPixelSizeX(_settings.pixelScaleX);
@@ -454,7 +463,7 @@ void WSClean::RunClean()
 			initializeMFSImageWeights();
 		
 		if(_settings.useIDG)
-			_gridder.reset(new IdgMsGridder());
+			_gridder.reset(new IdgMsGridder(_settings));
 		else
 			_gridder.reset(new WSMSGridder(&_imageAllocator, _settings.threadCount, _settings.memFraction, _settings.absMemLimit));
 		
@@ -534,7 +543,7 @@ ImageWeightCache* WSClean::createWeightCache()
 {
 	ImageWeightCache* cache = new ImageWeightCache(
 		_settings.weightMode,
-		_settings.untrimmedImageWidth, _settings.untrimmedImageHeight,
+		_settings.paddedImageWidth, _settings.paddedImageHeight,
 		_settings.pixelScaleX, _settings.pixelScaleY,
 		_settings.minUVInLambda, _settings.maxUVInLambda,
 		_settings.rankFilterLevel, _settings.rankFilterSize);
@@ -577,7 +586,7 @@ void WSClean::RunPredict()
 		if(_doReorder) performReordering(true);
 		
 		if(_settings.useIDG)
-			_gridder.reset(new IdgMsGridder());
+			_gridder.reset(new IdgMsGridder(_settings));
 		else
 			_gridder.reset(new WSMSGridder(&_imageAllocator, _settings.threadCount, _settings.memFraction, _settings.absMemLimit));
 	
@@ -690,7 +699,6 @@ void WSClean::runIndependentGroup(ImagingTable& groupTable)
 					for(size_t sGroupIndex=0; sGroupIndex!=groupTable.SquaredGroupCount(); ++sGroupIndex)
 					{
 						const ImagingTable sGroupTable = groupTable.GetSquaredGroup(sGroupIndex);
-						size_t currentChannelIndex = sGroupTable.Front().outputChannelIndex;
 						if(_settings.dftPrediction)
 						{
 							dftPredict(sGroupTable);
@@ -700,8 +708,8 @@ void WSClean::runIndependentGroup(ImagingTable& groupTable)
 								initializeCurMSProviders(sGroupTable[e]);
 								initializeImageWeights(sGroupTable[e]);
 			
-								imageMainNonFirst(sGroupTable[e].polarization, currentChannelIndex);
-								clearCurMSProviders();
+								imageMainNonFirst(sGroupTable[e]);
+								_currentPolMSes.clear();
 							}
 						}
 						else {
@@ -711,10 +719,17 @@ void WSClean::runIndependentGroup(ImagingTable& groupTable)
 								initializeCurMSProviders(sGroupTable[e]);
 								initializeImageWeights(sGroupTable[e]);
 			
-								predict(sGroupTable[e].polarization, currentChannelIndex);
+								predict(sGroupTable[e]);
+								_currentPolMSes.clear();
+							}
+							for(size_t e=0; e!=sGroupTable.EntryCount(); ++e)
+							{
+								prepareInversionAlgorithm(sGroupTable[e].polarization);
+								initializeCurMSProviders(sGroupTable[e]);
+								initializeImageWeights(sGroupTable[e]);
 								
-								imageMainNonFirst(sGroupTable[e].polarization, currentChannelIndex);
-								clearCurMSProviders();
+								imageMainNonFirst(sGroupTable[e]);
+								_currentPolMSes.clear();
 							} // end of polarization loop
 						}
 					} // end of joined channels loop
@@ -863,19 +878,18 @@ void WSClean::readEarlierModelImages(const ImagingTableEntry& entry)
 		std::string prefix = ImageFilename::GetPrefix(_settings, entry.polarization, entry.outputChannelIndex, entry.outputIntervalIndex, i==1);
 		FitsReader reader(prefix + "-model.fits");
 		Logger::Info << "Reading " << reader.Filename() << "...\n";
-		if(_settings.untrimmedImageWidth == 0 && _settings.untrimmedImageHeight == 0)
+		if(_settings.trimmedImageWidth == 0 && _settings.trimmedImageHeight == 0)
 		{
 			_settings.trimmedImageWidth = reader.ImageWidth();
 			_settings.trimmedImageHeight = reader.ImageHeight();
-			_settings.untrimmedImageWidth = (size_t) ceil(_settings.trimmedImageWidth * _settings.imagePadding);
-			_settings.untrimmedImageHeight = (size_t) ceil(_settings.trimmedImageHeight * _settings.imagePadding);
-			// Make the width and height divisable by four.
-			_settings.untrimmedImageWidth += (4-(_settings.untrimmedImageWidth%4))%4;
-			_settings.untrimmedImageHeight += (4-(_settings.untrimmedImageHeight%4))%4;
-			Logger::Debug << "Using image size of " << _settings.trimmedImageWidth << " x " << _settings.trimmedImageHeight << ", padded to " << _settings.untrimmedImageWidth << " x " << _settings.untrimmedImageHeight << ".\n";
+			_settings.RecalculatePaddedDimensions();
 		}
 		else if(reader.ImageWidth()!=_settings.trimmedImageWidth || reader.ImageHeight()!=_settings.trimmedImageHeight)
-			throw std::runtime_error("Inconsistent image size: dimensions of input image did not match.");
+		{
+			std::ostringstream msg;
+			msg << "Inconsistent image size: dimensions of input image did not match, input: " << reader.ImageWidth() << " x " << reader.ImageHeight() << ", specified: " << _settings.trimmedImageWidth << " x " << _settings.trimmedImageHeight;
+			throw std::runtime_error(msg.str());
+		}
 		
 		if(reader.PixelSizeX()==0.0 || reader.PixelSizeY()==0.0)
 			Logger::Warn << "Warning: input fits file misses the pixel size keywords.\n";
@@ -889,7 +903,11 @@ void WSClean::readEarlierModelImages(const ImagingTableEntry& entry)
 		// Here I require the pixel scale to be accurate enough so that the image is at most 1/10th pixel larger/smaller.
 		else if(std::fabs(reader.PixelSizeX() - _settings.pixelScaleX) * _settings.trimmedImageWidth > 0.1 * _settings.pixelScaleX ||
 			std::fabs(reader.PixelSizeY() - _settings.pixelScaleY) * _settings.trimmedImageHeight > 0.1 * _settings.pixelScaleY)
-			throw std::runtime_error("Inconsistent pixel size: pixel size of input image did not match.");
+		{
+			std::ostringstream msg;
+			msg << "Inconsistent pixel size: pixel size of input image did not match. Input: " << reader.PixelSizeX() << " x " << reader.PixelSizeY() << ", specified: " << _settings.pixelScaleX << " x " << _settings.pixelScaleY;
+			throw std::runtime_error(msg.str());
+		}
 		if(_settings.pixelScaleX == 0.0 || _settings.pixelScaleY == 0.0)
 		{
 			throw std::runtime_error("Could not determine proper pixel size. The input image did not provide proper pixel size values, and no or an invalid -scale was provided to WSClean");
@@ -940,9 +958,9 @@ void WSClean::predictGroup(const ImagingTable& imagingGroup)
 		initializeCurMSProviders(entry);
 		initializeImageWeights(entry);
 
-		predict(entry.polarization, 0);
+		predict(entry);
 		
-		clearCurMSProviders();
+		_currentPolMSes.clear();
 	} // end of polarization loop
 	
 	_imageAllocator.ReportStatistics();
@@ -951,13 +969,13 @@ void WSClean::predictGroup(const ImagingTable& imagingGroup)
 	_settings.prefixName = rootPrefix;
 }
 
-MSProvider* WSClean::initializeMSProvider(const ImagingTableEntry& entry, const MSSelection& selection, size_t filenameIndex, size_t dataDescId)
+std::unique_ptr<MSProvider> WSClean::initializeMSProvider(const ImagingTableEntry& entry, const MSSelection& selection, size_t filenameIndex, size_t dataDescId)
 {
 	PolarizationEnum pol = _settings.useIDG ? Polarization::Instrumental : entry.polarization;
 	if(_doReorder)
-		return new PartitionedMS(_partitionedMSHandles[filenameIndex], entry.msData[filenameIndex].bands[dataDescId].partIndex, pol, dataDescId);
+		return std::unique_ptr<MSProvider>(new PartitionedMS(_partitionedMSHandles[filenameIndex], entry.msData[filenameIndex].bands[dataDescId].partIndex, pol, dataDescId));
 	else
-		return new ContiguousMS(_settings.filenames[filenameIndex], _settings.dataColumnName, selection, pol, dataDescId, _settings.deconvolutionMGain != 1.0);
+		return std::unique_ptr<MSProvider>(new ContiguousMS(_settings.filenames[filenameIndex], _settings.dataColumnName, selection, pol, dataDescId));
 }
 
 void WSClean::initializeCurMSProviders(const ImagingTableEntry& entry)
@@ -970,9 +988,10 @@ void WSClean::initializeCurMSProviders(const ImagingTableEntry& entry)
 			MSSelection selection(_globalSelection);
 			if(selectChannels(selection, i, d, entry))
 			{
-				MSProvider* msProvider = initializeMSProvider(entry, selection, i, d);
-				_gridder->AddMeasurementSet(msProvider, selection);
-				_currentPolMSes.push_back(msProvider);
+				std::unique_ptr<MSProvider> msProvider(
+					initializeMSProvider(entry, selection, i, d));
+				_gridder->AddMeasurementSet(msProvider.get(), selection);
+				_currentPolMSes.emplace_back(std::move(msProvider));
 			}
 		}
 	}
@@ -987,21 +1006,14 @@ void WSClean::initializeMSProvidersForPB(const ImagingTableEntry& entry, Primary
 			MSSelection selection(_globalSelection);
 			if(selectChannels(selection, i, d, entry))
 			{
-				MSProvider* msProvider = initializeMSProvider(entry, selection, i, d);
-				pb.AddMS(msProvider, selection);
-				_currentPolMSes.push_back(msProvider);
+				std::unique_ptr<MSProvider> msProvider = initializeMSProvider(entry, selection, i, d);
+				pb.AddMS(msProvider.get(), selection);
+				_currentPolMSes.emplace_back(std::move(msProvider));
 			}
 		}
 	}
 }
 
-void WSClean::clearCurMSProviders()
-{
-	for(std::vector<MSProvider*>::iterator i=_currentPolMSes.begin(); i != _currentPolMSes.end(); ++i)
-		delete *i;
-	_currentPolMSes.clear();
-}
-
 void WSClean::runFirstInversion(ImagingTableEntry& entry)
 {
 	initializeCurMSProviders(entry);
@@ -1027,7 +1039,7 @@ void WSClean::runFirstInversion(ImagingTableEntry& entry)
 		_primaryBeam->SetPhaseCentre(ra, dec, dl, dm);
 		ImageFilename imageName = ImageFilename(entry.outputChannelIndex, entry.outputIntervalIndex);
 		_primaryBeam->MakeBeamImages(imageName, entry, _imageWeightCache.get(), _imageAllocator);
-		clearCurMSProviders();
+		_currentPolMSes.clear();
 		initializeCurMSProviders(entry);
 	}
 		
@@ -1037,7 +1049,7 @@ void WSClean::runFirstInversion(ImagingTableEntry& entry)
 		_modelImages.SetFitsWriter(writer);
 		_residualImages.SetFitsWriter(writer);
 		
-		imageMainFirst(entry.polarization, entry.outputChannelIndex);
+		imageMainFirst(entry);
 		
 		// If this was the first polarization of this channel, we need to set
 		// the info for this channel
@@ -1103,8 +1115,7 @@ void WSClean::runFirstInversion(ImagingTableEntry& entry)
 			}
 		}
 	}
-	
-	clearCurMSProviders();
+	_currentPolMSes.clear();
 }
 
 void WSClean::makeMFSImage(const string& suffix, size_t intervalIndex, PolarizationEnum pol, bool isImaginary, bool isPSF)
@@ -1279,7 +1290,7 @@ void WSClean::saveUVImage(const double* image, PolarizationEnum pol, const Imagi
 
 void WSClean::makeImagingTable(size_t outputIntervalIndex)
 {
-	std::set<OrderedChannel> channelSet;
+	std::set<ChannelInfo> channelSet;
 	double highestFreq = 0.0;
 	_msBands.assign(_settings.filenames.size(), MultiBandData());
 	for(size_t i=0; i!=_settings.filenames.size(); ++i)
@@ -1320,7 +1331,7 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex)
 		str << "Parameter '-channelsout' was set to an invalid value: " << _settings.channelsOut << " output channels requested, but combined in all specified measurement sets, there are only " << channelSet.size() << " unique channels.";
 		throw std::runtime_error(str.str());
 	}
-	_inputChannelFrequencies = std::vector<OrderedChannel>(channelSet.begin(), channelSet.end());
+	_inputChannelFrequencies = std::vector<ChannelInfo>(channelSet.begin(), channelSet.end());
 	
 	size_t joinedGroupIndex = 0, squaredGroupIndex = 0;
 	_imagingTable.Clear();
@@ -1356,7 +1367,7 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex)
 	_imagingTable.Print();
 }
 
-void WSClean::makeImagingTableEntry(const std::vector<OrderedChannel>& channels, size_t outIntervalIndex, size_t outChannelIndex, ImagingTableEntry& entry)
+void WSClean::makeImagingTableEntry(const std::vector<ChannelInfo>& channels, size_t outIntervalIndex, size_t outChannelIndex, ImagingTableEntry& entry)
 {
 	size_t startCh, width;
 	if(_settings.endChannel != 0)
@@ -1444,8 +1455,6 @@ void WSClean::fitBeamSize(double& bMaj, double& bMin, double& bPA, const double*
 			beamEstimate,
 			bMaj, bMin, bPA);
 	}
-	if(bMaj < 1.0) bMaj = 1.0;
-	if(bMin < 1.0) bMin = 1.0;
 	bMaj = bMaj*0.5*(_settings.pixelScaleX+_settings.pixelScaleY);
 	bMin = bMin*0.5*(_settings.pixelScaleX+_settings.pixelScaleY);
 }
