@@ -2,7 +2,7 @@
 #define WSCLEAN_SETTINGS_H
 
 #include "wstackinggridder.h"
-#include "inversionalgorithm.h"
+#include "measurementsetgridder.h"
 
 #include "../msselection.h"
 #include "../system.h"
@@ -21,11 +21,13 @@ public:
 	
 	void Validate() const;
 	
-	void Propogate() { setDimensions(); }
+	void Propogate() { RecalculatePaddedDimensions(); }
+	
+	void RecalculatePaddedDimensions();
 	
 	std::vector<std::string> filenames;
 	enum Mode { ImagingMode, PredictMode, RestoreMode } mode;
-	size_t untrimmedImageWidth, untrimmedImageHeight;
+	size_t paddedImageWidth, paddedImageHeight;
 	size_t trimmedImageWidth, trimmedImageHeight;
 	double imagePadding;
 	size_t widthForNWCalculation, heightForNWCalculation;
@@ -50,11 +52,14 @@ public:
 	WeightMode weightMode;
 	std::string prefixName;
 	bool smallInversion, makePSF, makePSFOnly, isWeightImageSaved, isUVImageSaved, isDirtySaved, isGriddingImageSaved;
+	bool writeImagingWeightSpectrumColumn;
 	bool dftPrediction, dftWithBeam;
 	std::string temporaryDirectory;
 	bool forceReorder, forceNoReorder, subtractModel, modelUpdateRequired, mfsWeighting;
 	bool normalizeForWeighting;
-	bool applyPrimaryBeam, reusePrimaryBeam, useDifferentialLofarBeam, savePsfPb, useIDG;
+	bool applyPrimaryBeam, reusePrimaryBeam, useDifferentialLofarBeam, savePsfPb;
+	bool useIDG, gridWithBeam;
+	enum IDGMode { IDG_DEFAULT, IDG_GPU, IDG_CPU, IDG_HYBRID } idgMode;
 	enum GridModeEnum gridMode;
 	enum MeasurementSetGridder::VisibilityWeightingMode visibilityWeightingMode;
 	double baselineDependentAveragingInWavelengths;
@@ -67,9 +72,9 @@ public:
 	double deconvolutionThreshold, deconvolutionGain, deconvolutionMGain;
 	bool autoDeconvolutionThreshold, autoMask;
 	double autoDeconvolutionThresholdSigma, autoMaskSigma;
-	bool rmsBackground;
-	double rmsBackgroundWindow;
-	enum RMSBackgroundMethod { RMSWindow, RMSAndMinimumWindow } rmsBackgroundMethod;
+	bool localRMS;
+	double localRMSWindow;
+	enum LocalRMSMethod { RMSWindow, RMSAndMinimumWindow } localRMSMethod;
 	bool saveSourceList;
 	size_t deconvolutionIterationCount, majorIterationCount;
 	bool allowNegativeComponents, stopOnNegativeComponents;
@@ -83,7 +88,7 @@ public:
 	
 	double deconvolutionBorderRatio;
 	std::string fitsDeconvolutionMask, casaDeconvolutionMask;
-	std::string rmsBackgroundImage;
+	std::string localRMSImage;
 	bool useMoreSaneDeconvolution, useIUWTDeconvolution, iuwtSNRTest;
 	std::string moreSaneLocation, moreSaneArgs;
 	ao::uvector<double> moreSaneSigmaLevels;
@@ -115,13 +120,12 @@ public:
 	
 private:
 	void checkPolarizations() const;
-	void setDimensions();
 };
 
 inline WSCleanSettings::WSCleanSettings() :
 	filenames(),
 	mode(ImagingMode),
-	untrimmedImageWidth(0), untrimmedImageHeight(0),
+	paddedImageWidth(0), paddedImageHeight(0),
 	trimmedImageWidth(0), trimmedImageHeight(0),
 	imagePadding(1.2),
 	widthForNWCalculation(0), heightForNWCalculation(0),
@@ -151,6 +155,7 @@ inline WSCleanSettings::WSCleanSettings() :
 	prefixName("wsclean"),
 	smallInversion(true), makePSF(false), makePSFOnly(false), isWeightImageSaved(false),
 	isUVImageSaved(false), isDirtySaved(true), isGriddingImageSaved(false),
+	writeImagingWeightSpectrumColumn(false),
 	dftPrediction(false), dftWithBeam(false),
 	temporaryDirectory(),
 	forceReorder(false), forceNoReorder(false),
@@ -162,6 +167,8 @@ inline WSCleanSettings::WSCleanSettings() :
 	useDifferentialLofarBeam(false),
 	savePsfPb(false),
 	useIDG(false),
+	gridWithBeam(false),
+	idgMode(IDG_DEFAULT),
 	gridMode(KaiserBesselKernel),
 	visibilityWeightingMode(MeasurementSetGridder::NormalVisibilityWeighting),
 	baselineDependentAveragingInWavelengths(0.0),
@@ -175,9 +182,9 @@ inline WSCleanSettings::WSCleanSettings() :
 	autoMask(false),
 	autoDeconvolutionThresholdSigma(0.0),
 	autoMaskSigma(0.0),
-	rmsBackground(false),
-	rmsBackgroundWindow(25.0),
-	rmsBackgroundMethod(RMSWindow),
+	localRMS(false),
+	localRMSWindow(25.0),
+	localRMSMethod(RMSWindow),
 	saveSourceList(false),
 	deconvolutionIterationCount(0),
 	majorIterationCount(20),
@@ -194,7 +201,7 @@ inline WSCleanSettings::WSCleanSettings() :
 	multiscaleConvolutionPadding(1.1),
 	multiscaleScaleList(),
 	multiscaleShapeFunction(MultiScaleTransforms::TaperedQuadraticShape),
-	deconvolutionBorderRatio(0.05),
+	deconvolutionBorderRatio(0.0),
 	fitsDeconvolutionMask(),
 	casaDeconvolutionMask(),
 	useMoreSaneDeconvolution(false),
